<?php

namespace DMG\AI_Tools;

/**
 * Actions for handling the DMG AI tools settings
 *
 * Class Settings
 *
 * @package MDT
 */
class Settings {

	public array $views;

	const FEATURE_NAME = 'DMG AI Tools';
	const PAGE_SLUG = 'dmg_ai_tools';
	const SETTINGS = [
		'dmg_ai_tools_url'               => [
			'label' => 'AI Service URL',
			'type'  => 'text'
		],
		'dmg_ai_tools_token'             => [
			'label' => 'Access Token',
			'type'  => 'text'
		],
		'dmg_ai_rewrite_prompt'          => [
			'label' => 'Rewrite prompt',
			'type'  => 'textarea'
		],
		'dmg_ai_tools_system'            => [
			'label' => 'System message',
			'type'  => 'textarea'
		],
		'dmg_ai_tools_temperature'       => [
			'label' => 'Temperature',
			'type'  => 'text'
		],
		'dmg_ai_tools_top_p'             => [
			'label' => 'Top P',
			'type'  => 'text'
		],
		'dmg_ai_tools_frequency_penalty' => [
			'label' => 'Frequency penalty',
			'type'  => 'text'
		],
		'dmg_ai_tools_presence_penalty'  => [
			'label' => 'Presence penalty',
			'type'  => 'text'
		],
		'dmg_ai_tools_max_tokens'        => [
			'label' => 'Max tokens',
			'type'  => 'text'
		],
		'dmg_ai_tools_paragraph_length'  => [
			'label' => 'Paragraph words length',
			'type'  => 'text'
		],
	];

	/**
	 * Settings constructor
	 */
	function __construct() {
		add_action( 'admin_menu', [ $this, 'add_page' ] );
		add_action( 'admin_init', [ $this, 'register_fields' ] );
	}

	/**
	 * Add Settings page
	 */
	public function add_page() {

		$top_menu                 = add_menu_page(
			self::FEATURE_NAME,                                             // page title
			self::FEATURE_NAME,                                             // menu text
			apply_filters( 'dmg_ai_tools_settings_cap', 'administrator' ),  // Only Administrators can access this section
			self::PAGE_SLUG,                                                // menu ID used to bind submenu items to this menu
			[ &$this, 'page_callback' ],                                    // callback to render the linked page
			'dashicons-embed-generic'                                       // menu icon
		);
		$this->views[ $top_menu ] = self::PAGE_SLUG;

		$settings_menu                 = add_submenu_page(
			self::PAGE_SLUG,
			'Settings',
			'Settings',
			'administrator',
			self::PAGE_SLUG,
			[ &$this, 'page_callback' ],
		);
		$this->views[ $settings_menu ] = self::PAGE_SLUG;

		return $this->views;
	}


	/**
	 * The settings page callback
	 */
	public function page_callback() {
		?>
		<div class="wrap">
			<h2><?php echo esc_html( self::FEATURE_NAME ); ?> settings</h2>
			<form method="post" action="options.php">
				<?php
				settings_fields( self::PAGE_SLUG );
				do_settings_sections( self::PAGE_SLUG );
				do_action( 'dmg_ai_tools_add_settings' );
				submit_button( 'Save Changes', 'primary', 'submit', true, [ 'id' => 'submit' ] );
				?>
			</form>
		</div>
		<?php
	}

	/**
	 * Register the fields to display in the settings page
	 */
	public static function register_fields() {

		add_settings_section( sprintf( '%s-general', self::PAGE_SLUG ), '', '__return_false', self::PAGE_SLUG );

		$settings = apply_filters( 'dmg_ai_tools_settings', self::SETTINGS );

		foreach ( $settings as $key => $value ) {
			self::register_setting( $key, $value['label'], $value['type'] );
		}

	}

	/**
	 * Add the settings field
	 *
	 * @param $id
	 * @param $label
	 * @param string $note
	 * @param string $form_field
	 */
	public static function register_setting( $id, $label, $type = 'text' ) {

		add_settings_field(
			$id,
			'<label for="' . esc_attr( $id ) . '">' . esc_attr( $label ) . '</label>',
			[ __CLASS__, $type ],
			self::PAGE_SLUG,
			sprintf( '%s-general', self::PAGE_SLUG ),
			[ $id ]
		);
		register_setting( self::PAGE_SLUG, $id );

	}

	/**
	 * Generate the input text field
	 *
	 * @param $args
	 */
	public static function text( $args ) {
		$value = get_option( $args[0] );
		printf( '<input type="text" id="%1$s" name="%1$s" value="%2$s" class="regular-text" />', esc_attr( $args[0] ), esc_attr( $value ) );
	}

	/**
	 * Generate the textarea field
	 *
	 * @param $args
	 */
	public static function textarea( $args ) {
		$value = get_option( $args[0] );
		printf( '<textarea id="%1$s" name="%1$s" rows="10" cols="50" class="large-text code">%2$s</textarea>', esc_attr( $args[0] ), esc_attr( $value ) );
	}
}

new Settings();
