import { __ } from '@wordpress/i18n';
import {InspectorControls, RichText, store as blockEditorStore, useBlockProps, BlockControls} from "@wordpress/block-editor";
import apiFetch from '@wordpress/api-fetch';
import { dispatch, select } from '@wordpress/data';
import { PanelBody, PanelRow, SelectControl, TextareaControl, __experimentalNumberControl as NumberControl, Button, ToolbarButton, Spinner, Tooltip} from "@wordpress/components";
import { createBlock } from '@wordpress/blocks';
import { useState, RawHTML } from '@wordpress/element';
import './editor.scss';
import './rules.js';

export default ( { attributes, setAttributes } ) => {

	const path = 'dmg-ai-tools/ask';
	const language = AIToolsOptions?.language ? AIToolsOptions.language : "British English";
	const allowedFormats = [];
	const [disabled, setDisabled] = useState(false);
	const [isRewriting, setIsRewriting] = useState(false);
	const [success, setSuccess] = useState(false);

	// Perform the request
	const sendRequest = () => {

		setDisabled(true);
		setIsRewriting(true);
		setSuccess(false);
		setAttributes({rewritten: ''});

		let system_message = attributes?.system_message ? attributes.system_message : AIToolsOptions?.system_message;
		if ( attributes.response_length ) system_message += `\n- Respond with a body content with a target length ${attributes.response_length} words`;
		system_message += `\n- Use ${language}`;

		let user_prompt = attributes?.rewrite_prompt ? attributes.rewrite_prompt : AIToolsOptions?.rewrite_prompt;
		user_prompt += ` using a ${attributes.tone} tone`;
		if (attributes.content) user_prompt += `\n\n${attributes.content}`;

		let data = {
			"messages": [
				{
					"role": "system",
					"content": system_message
				},
				{
					"role": "user",
					"content": user_prompt
				}
			],
			"temperature": attributes.temperature,
			"top_p": attributes.top_p,
			"frequency_penalty": attributes.frequency_penalty,
			"presence_penalty": attributes.presence_penalty,
			"max_tokens": attributes.max_tokens,
			"stop": null
		}

		performApiRequest(data).then(response_message => {

			let p_words_length = attributes?.p_words_length ? attributes.p_words_length : AIToolsOptions?.p_words_length;

			const jsonPattern = {
				"content": [
					`paragraph 1 (max ${p_words_length} words)`,
					`paragraph 2 (max ${p_words_length} words)`,
					`paragraph 3 (max ${p_words_length} words)`,
					`... (max ${p_words_length} words)`
				]
			};

			system_message += `\n- Use ${language}`;
			system_message += `\n- Use a ${attributes.tone} tone`;
			system_message += `\n- Respond in only one single JSON in this format: \n\n ${JSON.stringify(jsonPattern)}`;
			system_message += `\n- Every JSON element must have less than ${p_words_length} words`;

			user_prompt = attributes?.rewrite_prompt ? attributes.rewrite_prompt : AIToolsOptions?.rewrite_prompt;
			if (attributes.content) user_prompt += `\n\n${response_message}`;

			let data = {
				"messages": [
					{
						"role": "system",
						"content": system_message
					},
					{
						"role": "user",
						"content":  user_prompt
					}
				],
				"temperature": attributes.temperature,
				"top_p": attributes.top_p,
				"frequency_penalty": attributes.frequency_penalty,
				"presence_penalty": attributes.presence_penalty,
				"max_tokens": attributes.max_tokens,
				"stop": null
			}

			performApiRequest(data).then(response_message => {

				setDisabled(false);
				setIsRewriting(false);
				setSuccess(true);

				const json =  JSON.parse(response_message);
				const paragraphs = [];

				let rewritten_html = '';

				if ( json.content ) {
					json.content.forEach( paragraph => {
							rewritten_html += `<p>${paragraph}</p>`;
							paragraphs.push( paragraph );
					})
				}

				setAttributes({paragraphs: paragraphs});
				setAttributes({rewritten: rewritten_html});
			});
		});
	}

// Perform the Open AI API request
	const performApiRequest = async (data) => {

		return apiFetch({
			path: path,
			method: 'POST',
			data: data,
		}).then(response => {
			let response_message = response?.choices[0]?.message?.content;

			if (response_message.includes('__DMG_AI_ERROR__')) {
				setDisabled(false);
				setIsRewriting(false);
				setSuccess(false);
				setAttributes({rewritten: "I'm unable to generate new content or provide additional information. Please provide the content you'd like me to rephrase."});
			} else {
				return response_message;
			}
		}).catch(error => {
			setDisabled(false);
			setIsRewriting(false);
			setSuccess(false);

			if (error?.error?.message) {
				setAttributes({rewritten: error.error.message});
			} else {
				setAttributes({rewritten: 'A generic error occurred. Please try again'});
			}
		});
		;
	}

// Handle the transformation of the responce into gutenberg blocks
	const acceptTransformation = () => {
		let selectedBlock = select('core/block-editor').getSelectedBlock();
		let blocks = select('core/block-editor').getBlocks();
		let blockIndex = blocks.findIndex((block) => block.clientId === selectedBlock?.clientId);
		let newBlocks = [];

		if (blockIndex !== -1) {
			attributes?.paragraphs.forEach((paragraph) => {
				newBlocks.push(createBlock('core/paragraph', {content: paragraph}));
			});
			dispatch('core/block-editor').insertBlocks(newBlocks, blockIndex);
			dispatch('core/block-editor').removeBlocks([selectedBlock?.clientId]);


			// Keep track of the AI rewrite assigning a specific term
			let terms = select('core/editor').getEditedPostAttribute(AIToolsOptions?.ai_powered_taxonomy_name);
			terms.push(Number(AIToolsOptions?.ai_powered_term_id));
			dispatch('core/editor').editPost({[AIToolsOptions.ai_powered_taxonomy_name]:terms});
		}

	};

	return (
		<>
		<InspectorControls>
			<div class="ai-tools">
			<PanelBody
				title="Settings"
				initialOpen={true}>
				<PanelRow>
				<SelectControl
				label="Tone"
				value={ attributes.tone ||= 'informal' }
				options={[
						{ value: 'formal', label: 'Formal' },
						{ value: 'informal', label: 'Informal' },
						{ value: 'optimistic', label: 'Optimistic' },
						{ value: 'humorous', label: 'Humorous' },
						{ value: 'serious', label: 'Serious' },
						{ value: 'skeptical', label: 'Skeptical' },
						{ value: 'empathetic', label: 'Empathetic' },
						{ value: 'confident', label: 'Confident' },
						{ value: 'passionate', label: 'Passionate' },
						{ value: 'provocative', label: 'Provocative' }
					]}
				onChange={ (newValue) => setAttributes({ tone : newValue }) }
				/>
				<Tooltip text="Define the tone for the rewriting" position="right">
					<div class="tooltip dashicon dashicons dashicons-info-outline"></div>
				</Tooltip>
				</PanelRow>
				<PanelRow>
				<NumberControl
					isShiftStepEnabled={ true }
					label="Words Length"
					min = "0"
					value={ attributes.response_length }
					onChange={ (value) => setAttributes({ response_length:  Number(value) }) }
				/>
				<Tooltip text="Define the length of the rewritten content" position="right">
					<div class="tooltip dashicon dashicons dashicons-info-outline"></div>
				</Tooltip>
				</PanelRow>
				<PanelRow>
				<NumberControl
					isShiftStepEnabled={ true }
					label="P Words Length"
					min = "0"
					value={ attributes.p_words_length ||= AIToolsOptions?.p_words_length  }
					onChange={ (value) => setAttributes({ p_words_length:  Number(value) }) }
				/>
				<Tooltip text="Define how many words per paragraph" position="right">
				<div class="tooltip dashicon dashicons dashicons-info-outline"></div>
				</Tooltip>
				</PanelRow>
			</PanelBody>
			{AIToolsOptions?.is_advanced &&
			<PanelBody
				title="Advanced Settings"
				initialOpen={true}>
			<PanelRow>
				<TextareaControl
					label="User message"
					value={ attributes.rewrite_prompt ||= AIToolsOptions?.rewrite_prompt }
					onChange={ (value) => setAttributes({ rewrite_prompt: value }) }
				/>
				<Tooltip text="The user message represents the actual input or message from the user. It provides context and instructions." position="right">
					<div class="tooltip dashicon dashicons dashicons-info-outline"></div>
				</Tooltip>
				</PanelRow>
				<PanelRow>
					<TextareaControl
					label="System message"
					value={ attributes.system_message ||= AIToolsOptions?.system_message }
					onChange={ (value) => setAttributes({ system_message: value }) }
					/>
				<Tooltip text="Give the model instructions about how it should behave and any context it should reference when generating a response. You can tell the assistant what it should and shouldn’t answer, and tell it how to format responses." position="right">
					<div class="tooltip dashicon dashicons dashicons-info-outline"></div>
				</Tooltip>
				</PanelRow>
				<PanelRow>
				<NumberControl
					isShiftStepEnabled={ true }
					step = "0.01"
					label="temperature"
					max = "1"
					min = "0"
					value={ attributes.temperature ||= Number(AIToolsOptions?.temperature) }
					onChange={ (value) => setAttributes({ temperature: Number(value) }) }
				/>
				<Tooltip text="Controls randomness. Lowering the temperature means that the model will produce more repetitive and deterministic responses. Try adjusting temperature or Top P but not both." position="right">
					<div class="tooltip dashicon dashicons dashicons-info-outline"></div>
				</Tooltip>
				</PanelRow>
				<PanelRow>
				<NumberControl
					isShiftStepEnabled={ true }
					step = "0.01"
					label="Top P"
					max = "1"
					min = "0"
					value={ attributes.top_p ||= Number(AIToolsOptions?.top_p) }
					onChange={ (value) => setAttributes({ top_p: Number(value) }) }
				/>
				<Tooltip text="Similar to temperature, this controls randomness but uses a different method. Lowering Top P will narrow the model’s token selection to likelier tokens. Try adjusting temperature or Top P but not both." position="right">
					<div class="tooltip dashicon dashicons dashicons-info-outline"></div>
				</Tooltip>
				</PanelRow>
				<PanelRow>
				<NumberControl
					isShiftStepEnabled={ true }
					step = "0.01"
					label="Frequency penalty"
					max = "2"
					min = "0"
					value={ attributes.frequency_penalty ||= Number(AIToolsOptions?.frequency_penalty) }
					onChange={ (value) => setAttributes({ frequency_penalty: Number(value) }) }
				/>
				<Tooltip text="Reduce the chance of repeating a token proportionally based on how often it has appeared in the text so far. This decreases the likelihood of repeating the exact same text in a response." position="right">
					<div class="tooltip dashicon dashicons dashicons-info-outline"></div>
				</Tooltip>
				</PanelRow>
				<PanelRow>
				<NumberControl
					isShiftStepEnabled={ true }
					step = "0.01"
					label="Presence penalty"
					max = "2"
					min = "0"
					value={ attributes.presence_penalty ||= Number(AIToolsOptions?.presence_penalty) }
					onChange={ (value) => setAttributes({ presence_penalty: Number(value) }) }
				/>
				<Tooltip text="Reduce the chance of repeating any token that has appeared in the text at all so far. This increases the likelihood of introducing new topics in a response." position="right">
					<div class="tooltip dashicon dashicons dashicons-info-outline"></div>
				</Tooltip>
				</PanelRow>
				<PanelRow>
				<NumberControl
					isShiftStepEnabled={ true }
					label="Max Tokens"
					min = "0"
					value={ attributes.max_tokens ||= Number(AIToolsOptions?.max_tokens) }
					onChange={ (value) => setAttributes({ max_tokens: Number(value) }) }
				/>
				<Tooltip text="Set a limit on the number of tokens per model response. One token is roughly 4 characters for typical English text." position="right">
					<div class="tooltip dashicon dashicons dashicons-info-outline"></div>
				</Tooltip>
				</PanelRow>
			</PanelBody>
			}
			</div>
		</InspectorControls>
		<div { ...useBlockProps() }>
			<RichText
				placeholder={__("Rewrite this...", "dmg")}
				value={attributes.content}
				allowedFormats={allowedFormats}
				onChange={ (newValue) => {
						let newContent = newValue.replace(/<[^>]*>?/gm, '' );
						setAttributes({ content: newContent } );
						setAttributes({ response_length: newContent.split(" ").length } );
					}
				}
				/>
				<Button
					variant="secondary"
					disabled={ disabled }
					onClick={ sendRequest }>
						{isRewriting ? 'REWRITING...' : 'REWRITE' }
				</Button>
				{attributes.rewritten &&
				<div>
				{ RawHTML( { children: attributes.rewritten } ) }
				</div>
				}
				{success && !isRewriting &&
				<Button variant="secondary" onClick={ acceptTransformation }>ACCEPT</Button>
				}
		</div>

			<BlockControls>
				{attributes.rewritten && !isRewriting &&
				<ToolbarButton disabled={ disabled } icon={'update'} label="Try again" onClick={ sendRequest }>Try again</ToolbarButton>
				}
				{success &&
				<ToolbarButton icon={'saved'}  label="Accept" onClick={ acceptTransformation }>Accept</ToolbarButton>
				}
			</BlockControls>
	{isRewriting &&
		<BlockControls>
			<ToolbarButton disabled={ disabled } label="Rewriting..."><Spinner /> Rewriting...</ToolbarButton>
		</BlockControls>
		}
		</>
	);
}

